/**
 * 
 */
package mpi.eudico.client.annotator.recognizer.gui;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import mpi.eudico.client.annotator.recognizer.data.FileParam;
import mpi.eudico.client.annotator.recognizer.data.Param;
import mpi.eudico.client.annotator.util.ElanFileFilter;

/**
 * A panel to specify a file or folder for input for or output from a detector.
 *  
 * @author Han Sloetjes
 */
public class FileParamPanel extends AbstractParamPanel implements ActionListener {
	private boolean inputType;
	private int contentType;
	private boolean optional;
	private String initialPath;
	
	private JTextField fileField;
	private JButton browseButton;
	
	/**
	 * The name of the parameter.
	 * 
	 * @param paramName the name of the parameter
	 * @param inputType if true, this is for selecting an input file for the detector, otherwise
	 * a file or folder for output needs to be specified
	 * @param contentType the type of the content of the file, tier, timeseries, audio, video, auxiliary. 
	 * A constant from FileParam.
	 * @param optional if true this parameter does not have to be set
	 */
	public FileParamPanel(String paramName, String description, boolean inputType, int contentType, boolean optional) {
		super(paramName, description);
		this.inputType = inputType;
		this.contentType = contentType;
		this.optional = optional;
		initComponents();
	}
	
	/**
	 * Constructor accepting a FileParam as an argument.
	 * 
	 * @param param the file param object
	 */
	public FileParamPanel(FileParam param) {
		super(param);
		if (param != null) {
			inputType = (param.ioType == FileParam.IN);
			optional = param.optional;
			contentType = param.contentType;
			initComponents();
		}
	}

	/**
	 * Adds a textfield and a browse button to the panel.
	 */
	@Override
	protected void initComponents() {
		super.initComponents();
		fileField = new JTextField();
		fileField.setEditable(true);
		browseButton = new JButton("...");// or icon?
		browseButton.addActionListener(this);
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = 2;
		gbc.gridy = 1;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1.0;
		gbc.insets = new Insets(1, 1, 4, 1);
		add(fileField, gbc);
		gbc.gridx = 2;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		add(browseButton, gbc);
		
		if (inputType) {
			if (!optional) {
				descLabel.setText("*IN: " + description);
			} else {
				descLabel.setText("IN: " + description);
			}
		} else {
			if (!optional) {
				descLabel.setText("*OUT: " + description);
			} else {
				descLabel.setText("OUT: " + description);
			}
		}
	}


	/**
	 * Returns the path to the file or folder.
	 * 
	 * @see mpi.eudico.client.annotator.recognizer.gui.AbstractParamPanel#getParamValue()
	 */
	@Override
	protected Object getParamValue() {
		return fileField.getText();
	}

	/**
	 * Sets a previously stored value for the path.
	 * 
	 * @see mpi.eudico.client.annotator.recognizer.gui.AbstractParamPanel#setParamValue(java.lang.Object)
	 */
	@Override
	protected void setParamValue(Object value) {
		if (fileField != null && value instanceof String) {
			initialPath = (String) value;
			fileField.setText(initialPath);
		}

	}

	/**
	 * Returns whether this file is optional.
	 * 
	 * @return whether this file is optional
	 */
	public boolean isOptional() {
		return optional;
	}
	
	/**
	 * Only the browse button can currently be the source.
	 */
	public void actionPerformed(ActionEvent e) {
		JFileChooser chooser = null;
		if (initialPath != null) {
			chooser = new JFileChooser(initialPath);
		} else {
			chooser = new JFileChooser();
		}
		//setTitle?
		FileFilter ff = null;
		if (contentType == FileParam.CSV_TIER || contentType == FileParam.CSV_TS) {
			ff = ElanFileFilter.createFileFilter(ElanFileFilter.CSV_TYPE);
		} else if (contentType == FileParam.TIER || contentType == FileParam.TIMESERIES) {
			ff = ElanFileFilter.createFileFilter(ElanFileFilter.XML_TYPE);
		}
		if (ff != null) {
			chooser.addChoosableFileFilter(ff);
			chooser.setFileFilter(ff);
		}
		if (inputType) {
			if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
				File f = chooser.getSelectedFile();
				initialPath = f.getAbsolutePath();
				fileField.setText(initialPath);
			}
		} else {
			// output
			if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
				File f = chooser.getSelectedFile();
				initialPath = f.getAbsolutePath();
				fileField.setText(initialPath);
			}
		}
	}

}
