/**
 * 
 */
package mpi.eudico.client.annotator.recognizer.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.text.ParseException;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.JSpinner.DefaultEditor;

import mpi.eudico.client.annotator.recognizer.data.NumParam;
import mpi.eudico.client.annotator.recognizer.data.Param;

/**
 * A parameter panel for numerical parameters.
 * 
 * @author Han Sloetjes
 */
public class NumParamPanel extends AbstractParamPanel {
	private float min;
	private float max;
	private float initial;
	private JSlider slider; //a slider only supports integers
	private JSpinner spinner;
	private JLabel rangeLabel;
	
	/**
	 * Constructor with parameter identifier, min. value, max. and the default value.
	 * @param paramName the identifier
	 * @param min the minimal value
	 * @param max the maximal value
	 * @param initial initial value 
	 */
	public NumParamPanel(String paramName, String description, float min, float max, float initial) {
		super(paramName, description);
		this.min = min;
		this.max = max;
		if (this.min > this.max) {
			float tmp = this.max;
			this.max = this.min;
			this.min = tmp;
		}
		this.initial = initial;
		initComponents();
	}
	
	/**
	 * Constructor taking a NumParam as an argument.
	 * 
	 * @param param the num param object
	 */
	public NumParamPanel(NumParam param) {
		super(param);
		if (param != null) {
			min = param.min;
			max = param.max;
			if (min > max) {
				float tmp = max;
				max = min;
				min = tmp;
			}
			
			initial = param.def;// or current?
			initComponents();
		}
	}

	/**
	 * @see mpi.eudico.client.annotator.recognizer.gui.AbstractParamPanel#initComponents()
	 */
	@Override
	protected void initComponents() {
		super.initComponents();
		//add a slider or slider
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = 1;
		gbc.gridy = 1;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		gbc.insets = new Insets(1, 1, 4, 1);
		// hier... review the step size	
		float extent = max - min;
		float step = extent / 10;
		
		if (step < 1) {
			int num = 0;
			float nstep = step;
			while (true){
				nstep *= 10;
				num++;
				if (nstep >= 1) {
					break;
				}
			}
			if (nstep >=1 && nstep < 5) {
				nstep = 1;
			} else {
				nstep = 5;
			}
			step = (float) (nstep / (Math.pow(10, num)));
		} else {
			int num = 0;
			float nstep = step;
			while (true){
				nstep /= 10;
				num++;
				if (nstep <= 10) {
					break;
				}
			}
			if (nstep >=1 && nstep < 5) {
				nstep = 1;
			} else {
				nstep = 5;
			}
			step = (float) (nstep * (Math.pow(10, num)));;
		}
		
		spinner = new JSpinner(new SpinnerNumberModel(initial, min, max, step));
		spinner.setPreferredSize(new Dimension(spinner.getPreferredSize().width + 20, spinner.getPreferredSize().height));
		//slider = new JSlider((int) min, (int) max, (int) initial);// a slider only supports integers
		add(spinner, gbc);
		rangeLabel = new JLabel("(" + min + " - " + max + ")");
		rangeLabel.setFont(rangeLabel.getFont().deriveFont(10));
		gbc.gridx = 1;
		gbc.insets = new Insets(1, 6, 4, 1);
		add(rangeLabel, gbc);
	}

	/**
	 * @see mpi.eudico.client.annotator.recognizer.gui.AbstractParamPanel#getParamValue()
	 */
	@Override
	protected Object getParamValue() {
		// get the current value of the ui
		if (spinner != null) {
			Object value = spinner.getValue();// this is the model's value
			try {
				spinner.commitEdit();
				value = spinner.getValue();// maybe a new value
			} catch (ParseException pe) {
				 JComponent editor = spinner.getEditor();
				 if (editor instanceof DefaultEditor) {
				     ((DefaultEditor)editor).getTextField().setValue(value);// last known
				 }
			}
			
			return value;
		}
		
		return null;
	}

	/**
	 * Sets the initial value for the parameter.
	 * 
	 * @param value the initial value for this parameter
	 */
	protected void setParamValue(Object value) {
		if (value instanceof Float) {
			float nv = (Float) value;
			if (nv >= min && nv <= max) {
				initial = nv;
				// update ui
				if (spinner != null) {
					spinner.setValue(value);
				}
			}
		} else if (value instanceof String) {
			try {
				float nv = Float.parseFloat((String) value);
				
				if (nv >= min && nv <= max) {
					initial = nv;
					// update ui
					if (spinner != null) {
						spinner.setValue(new Float(nv));
					}
				}
			} catch (NumberFormatException nfe) {
				// ignore
			}
		}
		
	}

}
