package mpi.eudico.client.util;

import mpi.eudico.client.annotator.util.ClientLogger;

import mpi.eudico.server.corpora.clom.Transcription;

import mpi.util.TimeFormatter;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;

import java.text.DecimalFormat;

import java.util.List;


/**
 * Exports annotations of a selection of tiers to a DVD subtitle textformat.
 *
 * @author Han Sloetjes
 * @version 1.0
 */
public class Transcription2SubtitleText implements ClientLogger {
    /** the Windows CR+LF line break for SRT */
    final private String WIN_NEWLINE = "\r\n";

    /**
     * Constructor
     */
    public Transcription2SubtitleText() {
        super();
    }

    /**
     * Exports to .srt format file.  Provides support for a minimum duration
     * tries to solve overlaps.
     *
     * @param transcription the transcription
     * @param tierNames the tiers to include
     * @param exportFile the file to export to
     * @param encoding the file encoding, defaults to UTF-8
     * @param beginTime the (selection) begintime
     * @param endTime the (selection) end time
     * @param minimalDuration the minimal duration per subtitle
     *
     * @throws IOException any io exception
     */
    public void exportTiersSRT(Transcription transcription, String[] tierNames,
        File exportFile, String encoding, long beginTime, long endTime,
        int minimalDuration, long offset) throws IOException {
        if (exportFile == null) {
            LOG.severe("No export file specified");

            return;
        }

        SubtitleSequencer sequencer = new SubtitleSequencer();

        List allUnits = sequencer.createSequence(transcription, tierNames,
                beginTime, endTime, minimalDuration, offset, true);

        SubtitleUnit unit;
        FileOutputStream out = null;
        BufferedWriter writer = null;

        out = new FileOutputStream(exportFile);

        if (encoding != null) {
            try {
                writer = new BufferedWriter(new OutputStreamWriter(out, encoding));
            } catch (UnsupportedEncodingException uee) {
                LOG.warning("Encoding not supported: " + encoding);
                writer = new BufferedWriter(new OutputStreamWriter(out, "UTF-8"));
            }
        } else {
            writer = new BufferedWriter(new OutputStreamWriter(out, "UTF-8"));
        }

        for (int i = 0; i < allUnits.size(); i++) {
            unit = (SubtitleUnit) allUnits.get(i);
            writer.write(String.valueOf(i + 1)); // some apps don't accept index 0
            writer.write(WIN_NEWLINE);
            writer.write(TimeFormatter.toString(unit.getBegin()).replace('.',
                    ','));
            writer.write(" --> ");
            writer.write(TimeFormatter.toString(unit.getCalcEnd()).replace('.',
                    ','));
            writer.write(WIN_NEWLINE);

            if (unit.getValue() != null) {
                writer.write(unit.getValue().replace('\n', ' '));
            } else {
                for (int j = 0; j < unit.getValues().length; j++) {
                    writer.write(unit.getValues()[j].replace('\n', ' '));

                    if (j != (unit.getValues().length - 1)) {
                        writer.write(WIN_NEWLINE);
                    }
                }
            }

            writer.write(WIN_NEWLINE);
            writer.write(WIN_NEWLINE);
        }

        writer.close();
    }

    /**
     * Exports to .srt format file using the default file encoding, UTF-8.
     *
     * @param transcription the transcription
     * @param tierNames the tiers to include
     * @param exportFile the file to export to
     * @param beginTime the (selection) begintime
     * @param endTime the (selection) end time
     * @param minimalDuration the minimal duration per subtitle
     *
     * @throws IOException any io exception
     */
    public void exportTiersSRT(Transcription transcription, String[] tierNames,
        File exportFile, long beginTime, long endTime, int minimalDuration)
        throws IOException {
        exportTiersSRT(transcription, tierNames, exportFile, null, beginTime,
            endTime, minimalDuration, 0L);
    }

    /**
     * Exports to .srt format file, using default file encoding (UTF-8),
     * interval begin (0), interval end (Long.MAX_VALUE) and minimal duration
     * (0).
     *
     * @param transcription the transcription
     * @param tierNames the tiers to include
     * @param exportFile the file to export to
     *
     * @throws IOException any io exception
     */
    public void exportTiersSRT(Transcription transcription, String[] tierNames,
        File exportFile) throws IOException {
        exportTiersSRT(transcription, tierNames, exportFile, 0, Long.MAX_VALUE,
            0);
    }


    /** A formatter for two-digit sequences. */
    private static final DecimalFormat twoDigits = new DecimalFormat("00");

    /**
     * Reformats a standard timecode into the format expected in Spruce STL.
     *
     * @param time the time to convert
     * @param frameRate assumed frame rate; if negative, assume default
     * @return an STL-formatted timecode
     */
    private String toSTLTimecode(long time, double frameRate) {
        String tc = null;

        if (frameRate == 25.0) {
            // If PAL, return a PAL timecode.
            tc = TimeFormatter.toTimecodePAL(time);
        } else if (frameRate == 29.97) {
            // If NTSC (drop-frame), return a 30drop-SMPTE timecode.
            tc = TimeFormatter.toTimecodeNTSC(time);
        } else if (frameRate == 30.0) {
            // If this is NTSC (non-drop-frame), return an SMPTE timecode in
            // STL format, assuming a literal 29.27fps.  (This may seem odd,
            // but for DVD Studio Pro 4, using 30drop SMPTE with NTSC sources
            // produces synchronization drift).
            int fc = (int) ((time * 29.97) / 1000);
            String hours = twoDigits.format((((fc / 30) / 60) / 60) % 24);
            String minutes = twoDigits.format(((fc / 30) / 60) % 60);
            String seconds = twoDigits.format((fc / 30) % 60);
            String frames = twoDigits.format(fc % 30);
            tc = hours + ":" + minutes + ":" + seconds + "." + frames;
        } else {
            // Otherwise, default to PAL.
            tc = TimeFormatter.toTimecodePAL(time);
        }

        // Replace the final comma in the timecode with a period.
        tc = tc.substring(0, 8) + '.' + tc.substring(9);
        return tc;
    }

    /**
     * Exports to .stl format file.  Provides support for a minimum duration
     * tries to solve overlaps.
     *
     * @param transcription the transcription
     * @param tierNames the tiers to include
     * @param exportFile the file to export to
     * @param encoding the file encoding, defaults to UTF-8
     * @param beginTime the (selection) begintime
     * @param endTime the (selection) end time
     * @param minimalDuration the minimal duration per subtitle
     * @param frameRate the assumed frame rate (negative if assuming default)
     *
     * @throws IOException any io exception
     */
    public void exportTiersSTL(Transcription transcription, String[] tierNames,
        File exportFile, String encoding, long beginTime, long endTime,
        int minimalDuration, long offset, double frameRate) throws IOException {
        if (exportFile == null) {
            LOG.severe("No export file specified");
            return;
        }

        SubtitleSequencer sequencer = new SubtitleSequencer();

        List allUnits = sequencer.createSequence(transcription, tierNames,
                beginTime, endTime, minimalDuration, offset, true);

        SubtitleUnit unit;
        FileOutputStream out = null;
        BufferedWriter writer = null;

        out = new FileOutputStream(exportFile);

        if (encoding != null) {
            try {
                writer = new BufferedWriter(new OutputStreamWriter(out, encoding));
            } catch (UnsupportedEncodingException uee) {
                LOG.warning("Encoding not supported: " + encoding);
                writer = new BufferedWriter(new OutputStreamWriter(out, "UTF-8"));
            }
        } else {
            writer = new BufferedWriter(new OutputStreamWriter(out, "UTF-8"));
        }

        // Write the preamble.
        writer.write("$TapeOffset = False");
        writer.write(WIN_NEWLINE);

        for (int i = 0; i < allUnits.size(); i++) {
            unit = (SubtitleUnit) allUnits.get(i);

            writer.write(toSTLTimecode((long) unit.getBegin(), frameRate));
            writer.write(',');
            writer.write(toSTLTimecode((long) unit.getCalcEnd(), frameRate));
            writer.write(',');

            if (unit.getValue() != null) {
                writer.write(unit.getValue().replace('\n', '|'));
            } else {
                String[] values = unit.getValues();
                for (int j = 0; j < values.length; j++) {
                    writer.write(values[j].replace('\n', '|'));
                    
                    if (j < values.length - 1) {
                        writer.write('|');
                    }
                }
            }

            writer.write(WIN_NEWLINE);
        }

        writer.close();
    }

    /**
     * Exports to .stl format file using the default file encoding, UTF-8.
     *
     * @param transcription the transcription
     * @param tierNames the tiers to include
     * @param exportFile the file to export to
     * @param beginTime the (selection) begintime
     * @param endTime the (selection) end time
     * @param minimalDuration the minimal duration per subtitle
     *
     * @throws IOException any io exception
     */
    public void exportTiersSTL(Transcription transcription, String[] tierNames,
        File exportFile, long beginTime, long endTime, int minimalDuration)
        throws IOException {
        exportTiersSTL(transcription, tierNames, exportFile, null, beginTime,
            endTime, minimalDuration, 0L, -1.0);
    }

    /**
     * Exports to .stl format file, using default file encoding (UTF-8),
     * interval begin (0), interval end (Long.MAX_VALUE) and minimal duration
     * (0).
     *
     * @param transcription the transcription
     * @param tierNames the tiers to include
     * @param exportFile the file to export to
     *
     * @throws IOException any io exception
     */
    public void exportTiersSTL(Transcription transcription, String[] tierNames,
        File exportFile) throws IOException {
        exportTiersSTL(transcription, tierNames, exportFile, 0, Long.MAX_VALUE,
            0);
    }
}
